/*%
   ZTEX Firmware Kit for EZ-USB FX3 Microcontrollers
   Copyright (C) 2009-2017 ZTEX GmbH.
   http://www.ztex.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
%*/
/*
    GPIO functions.
*/    

#include <cyu3gpio.h>
//#include <ztex.c>

#ifndef _ZTEX_GPIO_C_
#define _ZTEX_GPIO_C_

uint8_t ztex_gpio_set_output(uint8_t num, CyBool_t val) {
    CyU3PGpioSimpleConfig_t cfg;

    cfg.outValue    = val;
    cfg.driveLowEn  = CyTrue;
    cfg.driveHighEn = CyTrue;
    cfg.inputEn     = CyFalse;
    cfg.intrMode    = CY_U3P_GPIO_NO_INTR;
    ZTEX_REC_RET( CyU3PGpioSetSimpleConfig( num, &cfg) );
    return 0;
}

void ztex_gpio_set (uint8_t num, CyBool_t val) {
    CyU3PReturnStatus_t status;
    status = CyU3PGpioSimpleSetValue(num, val);
    if ( status )  ZTEX_LOG("Error setting GPIO %d: %d", num, status);
}

CyBool_t ztex_gpio_get (uint8_t num) {
    CyBool_t val;
//    ZTEX_LOG("pin no, : %d", num);
    CyU3PReturnStatus_t status;
    status = CyU3PGpioSimpleGetValue(num, &val);
//    ZTEX_LOG("Reading GPIO %d: %d", num, status);
    if ( status )  ZTEX_LOG("Error reading GPIO %d: %d", num, status);
    return val;
}

uint8_t ztex_gpio_set_open_drain(uint8_t num, CyBool_t val) {
    CyU3PGpioSimpleConfig_t cfg;

    cfg.outValue    = val;
    cfg.driveLowEn  = CyTrue;
    cfg.driveHighEn = CyFalse;
    cfg.inputEn     = CyTrue;
    cfg.intrMode    = CY_U3P_GPIO_NO_INTR;
    ZTEX_REC_RET( CyU3PGpioSetSimpleConfig( num, &cfg) );
    return 0;
}

uint8_t ztex_gpio_set_open_source(uint8_t num, CyBool_t val) {
    CyU3PGpioSimpleConfig_t cfg;

    cfg.outValue    = val;
    cfg.driveLowEn  = CyFalse;
    cfg.driveHighEn = CyTrue;
    cfg.inputEn     = CyTrue;
    cfg.intrMode    = CY_U3P_GPIO_NO_INTR;
    ZTEX_REC_RET( CyU3PGpioSetSimpleConfig( num, &cfg) );
    return 0;
}

uint8_t ztex_gpio_set_input(uint8_t num) {
    CyU3PGpioSimpleConfig_t cfg;

    cfg.outValue    = CyFalse;
    cfg.driveLowEn  = CyFalse;
    cfg.driveHighEn = CyFalse;
    cfg.inputEn     = CyTrue;
    cfg.intrMode    = CY_U3P_GPIO_NO_INTR;
    ZTEX_REC_RET( CyU3PGpioSetSimpleConfig( num, &cfg) );
    return 0;
}


//button

#define CY_FX_BUTTON_GPIO                       (23)
#define CY_FX_BUTTON_GPIO_1                     (24)
#define CY_FX_BUTTON_GPIO_2                     (25)


#define CY_FX_APP_GPIO_INTR_CB_EVENT_FLAG     (1 << 1)



void
CyFxGpioIntrCb (
       uint8_t gpioId
       )
{
   CyBool_t gpioValue = CyFalse;
   CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

   ZTEX_LOG ("Entered gpio call back\r\n");

   if (gpioId == CY_FX_BUTTON_GPIO)
   {
       apiRetStatus = CyU3PGpioGetValue (gpioId, &gpioValue);
       ZTEX_LOG ( "Entered gpio call back %d \r\n",gpioValue);
       if (apiRetStatus == CY_U3P_SUCCESS)
       {
           /* Check status of the pin */
           if (gpioValue == CyFalse)
           {

           	Button_Press_Event=1;
//           	Button_Press_Event_1=0;
             // CyU3PEventSet(&glAppEvent, CY_FX_APP_GPIO_INTR_CB_EVENT_FLAG, CYU3P_EVENT_OR);
           }
       }
   }

   // second button Button_Press_Event_1
   if (gpioId == CY_FX_BUTTON_GPIO_1)
     {
         apiRetStatus = CyU3PGpioGetValue (24, &gpioValue);
         ZTEX_LOG ( "Entered gpio call back %d \r\n",gpioValue);
         if (apiRetStatus == CY_U3P_SUCCESS)
         {
             /* Check status of the pin */
             if (gpioValue == CyFalse)
             {

             	Button_Press_Event_1=1;
//             	Button_Press_Event=0;
               // CyU3PEventSet(&glAppEvent, CY_FX_APP_GPIO_INTR_CB_EVENT_FLAG, CYU3P_EVENT_OR);
             }
         }
     }

   // third button Button_Press_Event_2
   if (gpioId == CY_FX_BUTTON_GPIO_2)
     {
         apiRetStatus = CyU3PGpioGetValue (25, &gpioValue);
         ZTEX_LOG ( "Entered gpio call back %d \r\n",gpioValue);
         if (apiRetStatus == CY_U3P_SUCCESS)
         {
             /* Check status of the pin */
             if (gpioValue == CyFalse)
             {

             	Button_Press_Event_2=1;
//             	Button_Press_Event=0;
               // CyU3PEventSet(&glAppEvent, CY_FX_APP_GPIO_INTR_CB_EVENT_FLAG, CYU3P_EVENT_OR);
             }
         }
     }
}

void ztex_gpio_init() {
    CyU3PGpioClock_t cfg;
    CyU3PGpioSimpleConfig_t gpioConfig;

    cfg.fastClkDiv = 2;
    cfg.slowClkDiv = 0;
    cfg.simpleDiv  = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
    cfg.clkSrc     = CY_U3P_SYS_CLK;
    cfg.halfDiv    = 0;

    //ZTEX_REC( CyU3PGpioInit (&cfg, GPIO_INT_HANDLER) );
    ZTEX_REC( CyU3PGpioInit (&cfg, CyFxGpioIntrCb) );
   // ZTEX_LOG("CyU3PDeviceGpioOverride failed, error code = \n");
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
       apiRetStatus = CyU3PDeviceGpioOverride (17, CyTrue);
               if (apiRetStatus != 0)
               {
                   /* Error Handling */
                   ZTEX_LOG("CyU3PDeviceGpioOverride failed, error code = %d\n", apiRetStatus);
                   //CyFxAppErrorHandler(apiRetStatus);
               }

        apiRetStatus = CyU3PDeviceGpioOverride (18, CyTrue);
                   if (apiRetStatus != 0)
                   {
                       /* Error Handling */
                    ZTEX_LOG("CyU3PDeviceGpioOverride failed, error code = %d\n", apiRetStatus);
                       //CyFxAppErrorHandler(apiRetStatus);
                   }
	   apiRetStatus = CyU3PDeviceGpioOverride (23, CyTrue);
		  if (apiRetStatus != 0)
		  {
			  /* Error Handling */
		   ZTEX_LOG("CyU3PDeviceGpioOverride failed, error code = %d\n", apiRetStatus);
			  //CyFxAppErrorHandler(apiRetStatus);
		  }

	   apiRetStatus = CyU3PDeviceGpioOverride (24, CyTrue);
		  if (apiRetStatus != 0)
		  {
			  /* Error Handling */
		   ZTEX_LOG("CyU3PDeviceGpioOverride failed, error code = %d\n", apiRetStatus);
			  //CyFxAppErrorHandler(apiRetStatus);
		  }

       apiRetStatus = CyU3PDeviceGpioOverride (25, CyTrue);
		  if (apiRetStatus != 0)
		  {
			  /* Error Handling */
		   ZTEX_LOG("CyU3PDeviceGpioOverride failed, error code = %d\n", apiRetStatus);
			  //CyFxAppErrorHandler(apiRetStatus);
		  }

	   gpioConfig.outValue    = CyFalse;
	   gpioConfig.inputEn     = CyTrue;
	   gpioConfig.driveLowEn  = CyFalse;
	   gpioConfig.driveHighEn = CyFalse;
	   gpioConfig.intrMode    = CY_U3P_GPIO_INTR_BOTH_EDGE;

//	   apiRetStatus = CyU3PGpioSetSimpleConfig(23, &gpioConfig);
//	   if (apiRetStatus != CY_U3P_SUCCESS)
//	   {
//		   /* Error handling */
//		   ZTEX_LOG ("GPIO Config Failed, code=%d\r\n",apiRetStatus);
//		   CyFxAppErrorHandler(apiRetStatus);
//	   }
//
//
//
//	   apiRetStatus = CyU3PGpioSetSimpleConfig(24, &gpioConfig);
//	   if (apiRetStatus != CY_U3P_SUCCESS)
//	   {
//		   /* Error handling */
//		   ZTEX_LOG ("GPIO Config Failed, code=%d\r\n",apiRetStatus);
//		   CyFxAppErrorHandler(apiRetStatus);
//	   }
//
//
//
//	   apiRetStatus = CyU3PGpioSetSimpleConfig(25, &gpioConfig);
//	   if (apiRetStatus != CY_U3P_SUCCESS)
//	   {
//		   /* Error handling */
//		   ZTEX_LOG ("GPIO Config Failed, code=%d\r\n",apiRetStatus);
//		   CyFxAppErrorHandler(apiRetStatus);
//	   }



}

#endif // _ZTEX_GPIO_C_
